package ch.usi.inf.sape.tracer.agent;

import java.lang.instrument.Instrumentation;
import java.util.ArrayList;
import java.util.List;
import java.util.StringTokenizer;


public class Agent {

	public static void premain(String agentArgs, Instrumentation inst) {
		System.out.println("Agent:\tstarting (arguments: '"+agentArgs+"')");
		inst.addTransformer(new Transformer(ArgumentProcessor.processArgs(agentArgs)));
	}
	
	static class ArgumentProcessor{
		
		private static enum State {EXCLUDE, EXCEPTION, NOT_INITIALIZED};
		
		private static State current = State.NOT_INITIALIZED;
		
	public static AgentConfiguration processArgs(final String args){
		if (args == null)
			new AgentConfiguration(new String[0], new String[0]);
		
		StringTokenizer tokenizer = new StringTokenizer(args, ":");
		
		/*List of application classes that we would like to  
		  keep them uninstrumented. One could set additional
		  paths through agent argument -Xexcluded within
		  ant build script
		 */
		List<String> excludedClassPathsList = new ArrayList<String>();
		
		/*List of classes or packages that are already located
		  under any of our excluded packages, but we need them to 
		  be instrumented for some analytical purposes.
		  Additional paths can be set through agent
		  argument -Xexcept within ant build script.
		  */
		List<String> exceptionalClassPathList = new ArrayList<String>();
		
		while(tokenizer.hasMoreTokens()){
			String next = tokenizer.nextToken();
			switch (next){
			case "-Xexclude":
				current = State.EXCLUDE;
				break;
			case "-Xexcept":
				current = State.EXCEPTION;
				break;
			default:
				if (current == State.EXCEPTION){
					exceptionalClassPathList.add(next);
				}else if (current == State.EXCLUDE){
					excludedClassPathsList.add(next);
				}
				
				break;
			}
			
		}
		
		/**
		 * used for specifying some platform specific paths
		 * that need to be treated differently
		 */
		final String[] hardCodedExcludedList = new String[]{
				"org/eclipse/jdt/internal/"
				,"org/eclipse/jdt/core/"
		};
		
		final String[] excludedClassPaths = new String[hardCodedExcludedList.length + excludedClassPathsList.size()];
		System.arraycopy(hardCodedExcludedList, 0, excludedClassPaths, 0, hardCodedExcludedList.length);
		System.arraycopy(excludedClassPathsList.toArray(new String[0]), 0, excludedClassPaths, hardCodedExcludedList.length, excludedClassPathsList.size());

		
		/**
		 * used for specifying some platform specific paths
		 * that need to be treated differently
		 */
		final String[] hardCodedExceptionList = new String[]{
				"ch/usi/inf/sape/tracer/test/snippets"
				,"com/sun/tools/javac/jvm/"
				,"org/eclipse/jdt/internal/compiler/codegen"
				,"org/eclipse/jdt/internal/compiler/ast"
		};
		
		final String[] exceptionalClassPaths = new String[hardCodedExceptionList.length + exceptionalClassPathList.size()];
		System.arraycopy(hardCodedExceptionList, 0, exceptionalClassPaths, 0, hardCodedExceptionList.length);
		System.arraycopy(exceptionalClassPathList.toArray(new String[0]), 0, exceptionalClassPaths, hardCodedExceptionList.length, exceptionalClassPathList.size());

		
		return new AgentConfiguration(excludedClassPaths, exceptionalClassPaths);
		
	}
	
	}	
}