package ch.usi.inf.sape.tracer.analyzer;

import java.io.BufferedOutputStream;
import java.io.File;
import java.io.FileNotFoundException;
import java.io.FileOutputStream;
import java.io.PrintWriter;
import java.util.Collections;
import java.util.Map;
import java.util.Set;

public class InfectoJSONWriter {

	
	private final Map<String, Map<Integer, Integer>> sourceImpactMap;
	
	private final String destination;
	
	private int indent = 0;
	
	public InfectoJSONWriter(String destination, Map<String, Map<Integer, Integer>> sourceImpactMap) {
		this.sourceImpactMap = sourceImpactMap;
		this.destination = destination;
	}
	
	public void generateJsonFiles() throws FileNotFoundException{
		final String path = destination+"/infecto"; 
		new File(path).mkdir();
		
		Set<String> sources = sourceImpactMap.keySet();
		for (String source : sources) {
			if (source.equals("UNTRACED"))
				continue;
			
			Map<Integer, Integer> lineImpacts = sourceImpactMap.get(source);
			PrintWriter out = new PrintWriter(new BufferedOutputStream(new FileOutputStream(path+"/"+source+".impacts.json")));
			generateJsonFile(out, lineImpacts);
			out.close();
		}
	}
	
	private void generateJsonFile(PrintWriter out, Map<Integer, Integer> lineImpacts){
		out.println("[");
		indent++;
		
		int maxLine = Collections.max(lineImpacts.keySet());
		int maxValue = Collections.max(lineImpacts.values());
		for (int i = 0; i < maxLine; i++){
			int realSourceLine = i + 1;
			//codemirror uses 0 base for the line numbers
			if (lineImpacts.get(realSourceLine) == null){
				addTuple(0, maxValue, out);
			}
			else{
				addTuple(lineImpacts.get(realSourceLine), maxValue, out);
			}
			
			if (i != (maxLine - 1)){
				indent(out);
				out.println(",");
			}
			
		}
		indent--;
		out.println("\n]");
	}
		
	private void addTuple(int raw, int maxImpact, PrintWriter out){
		indent(out);
		out.println("{");
		indent++;
		
		indent(out);
		out.print("\"raw\" : " + raw + " ,");
		out.println("\"normalized\" : " + scaleImpact(raw, maxImpact)+"");
		
		indent--;
		
		indent(out);
		out.println("}");
	}
		
	public int scaleImpact(int impact, int max){
		return 255 - ((((255)*(impact))/(max)));
	}
	
	private void indent(PrintWriter out) {
        for (int i=0; i<indent; i++) {
            out.print("  ");
        }
    }


}
