package ch.usi.inf.sape.tracer.analyzer.cdg;

import java.io.FileOutputStream;
import java.io.IOException;
import java.io.PrintStream;
import java.util.HashMap;
import java.util.Map;

public class DotGraphVisitor<V,E> implements GraphVisitor<V,E>{

	private int nodeIndex;
	
	private final Map<V, String> legalNodeNames = new HashMap<>();
	
	private final PrintStream out;
	
	private final Graph<V,E> g;
	
	public DotGraphVisitor(String fileName, Graph<V,E> g) throws IOException{
		out = new PrintStream(new FileOutputStream(fileName));
		this.g = g;
		out.println("digraph CFG {");
	}

	
	@Override
	public void visit(Vertex<V> v) {
		String legalNodeName = "bb" + nodeIndex;
		legalNodeNames.put(v.getLabel(), legalNodeName);
		out.println(legalNodeName + " [label=\"" + v.getVertexContents().getID()+
				"\n"+ v.getVertexContents() + "\"" + (v.equals(g.getRoot()) ? "style= filled color = green" : "") + "]");
		nodeIndex++;

	}

	@Override
	public void visit(Edge<V, E> edge) {
		out.print(legalNodeNames.get(edge.origin()) + "->" + legalNodeNames.get(edge.target()));
		if (edge.getLabel() != null && edge.getLabel().equals("EX"))
			out.print(" [style = dotted]");
		out.println();
	}

	@Override
	public void finalizeVisiting() {
		out.println("}");
		out.close();
	}
}
