package ch.usi.inf.sape.tracer.analyzer.locations;

import ch.usi.inf.sape.tracer.analyzer.TraceElementVisitor;
import ch.usi.inf.sape.tracer.analyzer.VisitorException;

public class ArrayElementMemoryLocation extends MemoryLocation{
	
	private final ArrayMetaData array;
	
	private final int index;

	public ArrayElementMemoryLocation(ArrayMetaData array, int index) {
		this.array = array;
		this.index = index;
	}

	/**
	 * @return the index
	 */
	public int getIndex() {
		return index;
	}

	@Override
	public String toString() {
		return array+"[" + this.index + "]";
	}

	@Override
	public int getPresentationOffset() {
		return index;
	}

	@Override
	public MemorySpaceType getSpaceType() {
		return MemorySpaceType.HEAP_SPACE;
	}

	@Override
	public ArrayMetaData getContainer() {
		return array;
	}

	@Override
	public boolean isStackSlot() {
		return false;
	}

	@Override
	public String getName() {
		return this.toString();
	}

	@Override
	public LocationType getLocationType() {
		return LocationType.ARRAY_ELEMENT;
	}

	@Override
	public void accept(TraceElementVisitor visitor) throws VisitorException {
		visitor.visit(this);
	}

	@Override
	public int hashCode() {
		final int prime = 31;
		int result = 1;
		result = prime * result + ((array == null) ? 0 : array.hashCode());
		result = prime * result + index;
		return result;
	}

	@Override
	public boolean equals(Object obj) {
		if (this == obj)
			return true;
		if (obj == null)
			return false;
		if (getClass() != obj.getClass())
			return false;
		ArrayElementMemoryLocation other = (ArrayElementMemoryLocation) obj;
		if (array == null) {
			if (other.array != null)
				return false;
		} else if (!array.equals(other.array))
			return false;
		if (index != other.index)
			return false;
		return true;
	}

}
